@@ -5,7 +5,7 @@ imdbpy2sql.py script.
 This script puts the data of the plain text data files into a
 SQL database.
 
-Copyright 2005-2006 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2007 Davide Alberani <da@erlug.linux.it>
                2006 Giuseppe "Cowo" Corbelli <cowo --> lugbs.linux.it>
 
 This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,9 @@ from gzip import GzipFile
 from types import UnicodeType
 
 from sqlobject import *
+from sqlobject.sqlbuilder import ISNOTNULL
 
-from imdb.parser.sql import soundex
+from imdb.parser.sql import soundex, get_movie_data
 from imdb.parser.sql.dbschema import *
 from imdb.utils import analyze_title, analyze_name, \
         build_name, build_title, normalizeName, _articles
@@ -38,6 +39,11 @@ from imdb.parser.local.movieParser import _bus, _ldk, _lit, _links_sect
 from imdb.parser.local.personParser import _parseBiography
 from imdb._exceptions import IMDbParserError
 
+_articles = list(_articles)
+for i, art in enumerate(_articles):
+    if not isinstance(art, unicode): continue
+    _articles[i] = art.encode('utf_8')
+
 re_nameImdbIndex = re.compile(r'\(([IVXLCDM]+)\)')
 
 HELP = """imdbpy2sql.py usage:
@@ -200,9 +206,9 @@ def title_soundex(title):
     article is pruned.  It assumes to receive a title without year/imdbIndex
     or kind indications, but just the title string, as the one in the
     analyze_title(title)['title'] value."""
-    title = unicode(title, 'utf_8')
+    ##if not isinstance(title, unicode): title = unicode(title, 'utf_8')
     # Prune non-ascii chars from the string.
-    title = title.encode('ascii', 'replace')
+    ##title = title.encode('ascii', 'replace')
     if not title: return None
     ts = title.split(', ')
     # Strip the ending article, if any.
@@ -219,9 +225,9 @@ def name_soundexes(name):
     from the first one.
     The third is the soundex of the surname, if different from the
     other two values."""
-    name = unicode(name, 'utf_8')
+    ##if not isinstance(name, unicode): name = unicode(name, 'utf_8')
     # Prune non-ascii chars from the string.
-    name = name.encode('ascii', 'ignore')
+    ##name = name.encode('ascii', 'ignore')
     if not name: return (None, None, None)
     s1 = soundex(name)
     name_normal = normalizeName(name)
@@ -432,9 +438,9 @@ class _BaseCache(dict):
                 self._tmpDict.clear()
             except OperationalError, e:
                 # Dataset too large; split it in two and retry.
-                ## new code!
-                ## the same class instance (self) is used, instead of
-                ## creating two separated objects.
+                # XXX: new code!
+                # the same class instance (self) is used, instead of
+                # creating two separated objects.
                 _recursionLevel += 1
                 self._flushing = 0
                 firstHalf = {}
@@ -492,10 +498,11 @@ class _BaseCache(dict):
 def fetchsome(curs, size=20000):
     """Yes, I've read the Python Cookbook! :-)"""
     while 1:
-        res = CURS.fetchmany(size)
+        res = curs.fetchmany(size)
         if not res: break
         for r in res: yield r
 
+
 class MoviesCache(_BaseCache):
     """Manage the movies list."""
     className = 'MoviesCache'
@@ -503,11 +510,11 @@ class MoviesCache(_BaseCache):
 
     def __init__(self, *args, **kwds):
         _BaseCache.__init__(self, *args, **kwds)
-        self.episodesYear = {}
+        self.movieYear = {}
         self.sqlstr, self.converter = createSQLstr(Title, ('id', 'title',
                                     'imdbIndex', 'kindID', 'productionYear',
                                     'phoneticCode', 'episodeOfID',
-                                    'seasonNr', 'episodeNr'))
+                                    'seasonNr', 'episodeNr', 'seriesYears'))
 
     def populate(self):
         print ' * POPULATING %s...' % self.className
@@ -520,9 +527,9 @@ class MoviesCache(_BaseCache):
         episodeofidCol = colName(Title, 'episodeOfID')
         seasonNrCol = colName(Title, 'seasonNr')
         episodeNrCol = colName(Title, 'episodeNr')
-        sqlPop = 'SELECT %s, %s, %s, %s, %s, %s, %s, %s FROM %s;' % (movieidCol,
-                    titleCol, kindidCol, yearCol, imdbindexCol,
-                    episodeofidCol, seasonNrCol, episodeNrCol, titleTbl)
+        sqlPop = 'SELECT %s, %s, %s, %s, %s, %s, %s, %s FROM %s;' % \
+                (movieidCol, titleCol, kindidCol, yearCol, imdbindexCol,
+                episodeofidCol, seasonNrCol, episodeNrCol, titleTbl)
         CURS.execute(sqlPop)
         _oldcacheValues = Title.sqlmeta.cacheValues
         Title.sqlmeta.cacheValues = False
@@ -542,7 +549,8 @@ class MoviesCache(_BaseCache):
                 if series_d['year'] is None: del series_d['year']
                 else: series_d['year'] = str(series_d['year'])
                 mdict['episode of'] = series_d
-            title = build_title(mdict, canonical=1, ptdf=1)
+            title = build_title(mdict, canonical=1, ptdf=1,
+                                _emptyString='')
             dict.__setitem__(self, title, x[0])
         self.counter = counter(Title.select().count() + 1)
         Title.sqlmeta.cacheValues = _oldcacheValues
@@ -556,7 +564,7 @@ class MoviesCache(_BaseCache):
         tmpDictiter = self._tmpDict.iteritems
         for k, v in tmpDictiter():
             try:
-                t = analyze_title(k)
+                t = analyze_title(k, _emptyString='')
             except IMDbParserError:
                 if k and k.strip():
                     print 'WARNING %s._toDB() invalid title:' % self.className,
@@ -567,18 +575,21 @@ class MoviesCache(_BaseCache):
             kind = tget('kind')
             if kind == 'episode':
                 #series title
-                stitle = build_title(tget('episode of'), canonical=1)
+                stitle = build_title(tget('episode of'), canonical=1,
+                                    _emptyString='')
                 episodeOf = self.addUnique(stitle)
                 del t['episode of']
-                year = self.episodesYear.get(v)
+                year = self.movieYear.get(v)
                 if year is not None:
                     try: t['year'] = int(year)
                     except ValueError: pass
+            elif kind in ('tv series', 'tv mini series'):
+                t['series years'] = self.movieYear.get(v)
             title = tget('title')
             soundex = title_soundex(title)
             lapp((v, title, tget('imdbIndex'), KIND_IDS[kind],
                     tget('year'), soundex, episodeOf,
-                    tget('season'), tget('episode')))
+                    tget('season'), tget('episode'), tget('series years')))
         self._runCommand(l)
 
     def _runCommand(self, dataList):
@@ -777,16 +788,16 @@ def readMovieList():
         line_d = unpack(line, ('title', 'year'))
         title = line_d['title']
         yearData = None
-        # Collect 'year' column for tv series' episodes.
-        if title[-1:] == '}':
-            yearData = [('episodesYear', line_d['year'])]
+        # Collect 'year' column for tv "series years" and episodes' year.
+        if title[0] == '"':
+            yearData = [('movieYear', line_d['year'])]
         mid = CACHE_MID.addUnique(title, yearData)
         if count % 10000 == 0:
             print 'SCANNING movies:', _(title),
             print '(movieID: %s)' % mid
         count += 1
     CACHE_MID.flush()
-    CACHE_MID.episodesYear.clear()
+    CACHE_MID.movieYear.clear()
     mdbf.close()
 
 
@@ -815,9 +826,18 @@ def doCast(fp, roleid, rolename):
         for item in ll[1:]:
             if not item: continue
             if item[0] == '[':
-                role = item[1:-1]
+                role = item[1:].rstrip(']')
+                if role[-1:] == ')':
+                    nidx = role.find('(')
+                    if nidx != -1:
+                        note = role[nidx:]
+                        role = role[:nidx].rstrip()
+                        if not role: role = None
             elif item[0] == '(':
-                note = item
+                if note is None:
+                    note = item
+                else:
+                    note = '%s %s' % (note, item)
             elif item[0] == '<':
                 textor = item[1:-1]
                 try:
@@ -923,7 +943,7 @@ class AkasMoviesCache(MoviesCache):
             # id of the referred title.
             original_title_id = self.ids.get(the_id)
             new_item = [the_id, original_title_id]
-            new_item += item[1:]
+            new_item += item[1:-1]
             new_item.append(self.notes.get(the_id))
             new_dataListapp(tuple(new_item))
         new_dataList.reverse()
@@ -1329,7 +1349,7 @@ def getRating():
 
 
 def getTopBottomRating():
-    """Movie's rating, scanning for top 250 and bottom 100."""
+    """Movie's rating, scanning for top 250 and bottom 10."""
     for what in ('top 250 rank', 'bottom 10 rank'):
         if what == 'top 250 rank': st = RAT_TOP250_START
         else: st = RAT_BOT10_START
@@ -1407,7 +1427,6 @@ CACHE_MID = MoviesCache()
 CACHE_PID = PersonsCache()
 
 
-
 def _cmpfunc(x, y):
     """Sort a list of tuples, by the length of the first item (in reverse)."""
     lx = len(x[0])
@@ -1441,54 +1460,163 @@ def readConstants():
         CCAST_TYPES[x.kind] = x.id
 
 
+def notNULLimdbID(cls):
+    """Return a list of dictionaries for titles or names for which a
+    imdbID is present in the database."""
+    if cls is Title: cname = 'movies'
+    else: cname = 'people'
+    print 'SAVING imdbID values for %s...' % cname,
+    sys.stdout.flush()
+    try:
+        tons = cls.select(ISNOTNULL(cls.q.imdbID))
+    except:
+        print 'SKIPPING: no data.'
+        return []
+    results = []
+    _kdict = {}
+    try:
+        for x in KindType.select():
+            _kdict[x.id] = x.kind
+    except:
+        print 'SKIPPING: no data.'
+        return []
+    for t in tons:
+        if cls is Title:
+            md = get_movie_data(t.id, _kdict)
+        else:
+            md = {'name': t.name}
+            if t.imdbIndex is not None:
+                md['imdbIndex'] = t.imdbIndex
+        md['imdbID'] = t.imdbID
+        results.append(md)
+    print 'DONE! (%d entries)' % len(results)
+    return results
+
+
+def restoreImdbID(tons, cls):
+    """Restore imdbID for movies or people."""
+    if cls is Title:
+        CACHE = CACHE_MID
+        cname = 'movies'
+    else:
+        CACHE = CACHE_PID
+        cname = 'people'
+    print 'RESTORING imdbID values for %s...' % cname,
+    sys.stdout.flush()
+    count = 0
+    for t in tons:
+        if cls is Title:
+            t_str = build_title(t, canonical=1, ptdf=1)
+        else:
+            t_str = build_name(t, canonical=1)
+        t_str = t_str.encode('utf_8')
+        db_mopID = CACHE.get(t_str)
+        if db_mopID is None:
+            continue
+        try:
+            mop_in_db = cls.get(db_mopID)
+            try:
+                mop_in_db.imdbID = t['imdbID']
+            except:
+                continue
+        except SQLObjectNotFound:
+            continue
+        count += 1
+    print 'DONE! (restored %d entries out of %d)' % (count, len(tons))
+
+
 # begin the iterations...
 def run():
     print 'RUNNING imdbpy2sql.py'
+
+    # Storing imdbIDs for movies and persons.
+    try:
+        movies_imdbIDs = notNULLimdbID(Title)
+    except:
+        movies_imdbIDs = []
+        print 'WARNING: failed to read imdbIDs for movies'
+    try:
+        people_imdbIDs = notNULLimdbID(Name)
+    except:
+        people_imdbIDs = []
+        print 'WARNING: failed to read imdbIDs for people'
+
     # Truncate the current database.
     print 'DROPPING current database...',
     sys.stdout.flush()
     dropTables()
-    print 'done!'
+    print 'DONE!'
+
+    # Rebuild the database structure.
     print 'CREATING new tables...',
     sys.stdout.flush()
     createTables()
-    readConstants()
-    print 'done!'
+    print 'DONE!'
     t('dropping and recreating the database')
 
+    # Read the constants.
+    readConstants()
+
     # Populate the CACHE_MID instance.
     readMovieList()
+    # Comment readMovieList() and uncomment the following two lines
+    # to keep the current info in the name and title tables.
     ##CACHE_MID.populate()
     ##CACHE_PID.populate()
     t('readMovieList()')
 
-
-    # actors, actresses, directors, ....
+    # actors, actresses, producers, writers, cinematographers, composers,
+    # costume-designers, directors, editors, miscellaneous,
+    # production-designers.
     castLists()
 
+    # Aka names and titles.
     doAkaNames()
     t('doAkaNames()')
     doAkaTitles()
     t('doAkaTitles()')
 
+    # alternate-versions, goofs, crazy-credits, quotes, soundtracks, trivia.
     doMinusHashFiles()
     t('doMinusHashFiles()')
 
+    # biographies, business, laserdisc, literature, mpaa-ratings-reasons, plot.
     doNMMVFiles()
 
+    # certificates, color-info, countries, distributors, genres, keywords,
+    # language, locations, miscellaneous-companies, production-companies,
+    # running-times, sound-mix, special-effects-companies, technical,
+    # release-dates.
     doMiscMovieInfo()
+    # movie-links.
     doMovieLinks()
     t('doMovieLinks()')
 
+    # ratings.
     getRating()
     t('getRating()')
+    # taglines.
     getTaglines()
     t('getTaglines()')
+    # ratings (top 250 and bottom 10 movies).
     getTopBottomRating()
     t('getTopBottomRating()')
+    # complete-cast, complete-crew.
     completeCast()
     t('completeCast()')
 
+    # Restoring imdbIDs for movies and persons.
+    try:
+        restoreImdbID(movies_imdbIDs, Title)
+        del movies_imdbIDs
+    except:
+        print 'WARNING: failed to restore imdbIDs for movies'
+    try:
+        restoreImdbID(people_imdbIDs, Name)
+        del people_imdbIDs
+    except:
+        print 'WARNING: failed to restore imdbIDs for people'
+
     # Flush caches.
     CACHE_MID.flush()
     CACHE_MID.clear()
@@ -1501,6 +1629,7 @@ def run():
 
     print 'building database indexes (this may take a while)'
     sys.stdout.flush()
+    # Build database indexes.
     createIndexes()
     t('createIndexes()')
 
