@@ -24,6 +24,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
+import sys
 from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
@@ -41,17 +42,20 @@ from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         dvd_parser, rec_parser, externalrev_parser, \
                         newsgrouprev_parser, misclinks_parser, \
                         soundclips_parser, videoclips_parser, news_parser, \
-                        photosites_parser, amazonrev_parser, guests_parser, \
-                        business_parser, sales_parser, episodes_parser, \
-                        eprating_parser, movie_faqs_parser, airing_parser
+                        photosites_parser, amazonrev_parser, business_parser, \
+                        literature_parser, sales_parser, episodes_parser, \
+                        eprating_parser, movie_faqs_parser, airing_parser, \
+                        synopsis_parser, parentsguide_parser
 from searchMovieParser import search_movie_parser
 from personParser import maindetails_parser, bio_parser, \
                         otherworks_parser, person_awards_parser, \
                         person_officialsites_parser, publicity_parser, \
-                        agent_parser, person_series_parser
+                        agent_parser, person_series_parser, \
+                        person_genres_parser, person_keywords_parser
 from searchPersonParser import search_person_parser
 from utils import ParserBase
 
+PY_VERSION = sys.version_info[:2]
 
 # Misc URLs
 imdbURL_movie = 'http://akas.imdb.com/title/tt%s/'
@@ -59,23 +63,55 @@ imdbURL_person = 'http://akas.imdb.com/name/nm%s/'
 imdbURL_search = 'http://akas.imdb.com/find?%s'
 
 # The cookies for the "adult" search.
-# Please don't mess with this account.
-_cookie_id = 'boM2bYxz9MCsOnH9gZ0S9QHs12NWrNdApxsls1Vb5/NGrNdjcHx3dUas10UASoAjVEvhAbGagERgOpNkAPvxdbfKwaV2ikEj9SzXY1WPxABmDKQwdqzwRbM+12NSeJFGUEx3F8as10WwidLzVshDtxaPIbP13NdjVS9UZTYqgTVGrNcT9vyXU1'
-_cookie_uu = '3M3AXsquTU5Gur/Svik+ewflPm5Rk2ieY3BIPlLjyK3C0Dp9F8UoPgbTyKiGtZp4x1X+uAUGKD7BM2g+dVd8eqEzDErCoYvdcvGLvVLAen1y08hNQtALjVKAe+1hM8g9QbNonlG1/t4S82ieUsBbrSIQbq1yhV6tZ6ArvSbA7rgHc8n5AdReyAmDaJ5Wm/ee3VDoCnGj/LlBs2ieUZNorhHDKK5Q=='
+# Please don't mess with these account.
+# Old 'IMDbPY' account.
+_old_cookie_id = 'boM2bYxz9MCsOnH9gZ0S9QHs12NWrNdApxsls1Vb5/NGrNdjcHx3dUas10UASoAjVEvhAbGagERgOpNkAPvxdbfKwaV2ikEj9SzXY1WPxABmDKQwdqzwRbM+12NSeJFGUEx3F8as10WwidLzVshDtxaPIbP13NdjVS9UZTYqgTVGrNcT9vyXU1'
+_old_cookie_uu = '3M3AXsquTU5Gur/Svik+ewflPm5Rk2ieY3BIPlLjyK3C0Dp9F8UoPgbTyKiGtZp4x1X+uAUGKD7BM2g+dVd8eqEzDErCoYvdcvGLvVLAen1y08hNQtALjVKAe+1hM8g9QbNonlG1/t4S82ieUsBbrSIQbq1yhV6tZ6ArvSbA7rgHc8n5AdReyAmDaJ5Wm/ee3VDoCnGj/LlBs2ieUZNorhHDKK5Q=='
+# New 'IMDbPYweb' account.
+_cookie_id = 'rH1jNAkjTlNXvHolvBVBsgaPICNZbNdjVjzFwzas9JRmusdjVoqBs/Hs12NR+1WFxEoR9bGKEDUg6sNlADqXwkas12N131Rwdb+UQNGKN8PWrNdjcdqBQVLq8mbGDHP3hqzxhbD692NQi9D0JjpBtRaPIbP1zNdjUOqENQYv1ADWrNcT9vyXU1'
+_cookie_uu = 'su4/m8cho4c6HP+W1qgq6wchOmhnF0w+lIWvHjRUPJ6nRA9sccEafjGADJ6hQGrMd4GKqLcz2X4z5+w+M4OIKnRn7FpENH7dxDQu3bQEHyx0ZEyeRFTPHfQEX03XF+yeN1dsPpcXaqjUZAw+lGRfXRQEfz3RIX9IgVEffdBAHw2wQXyf9xdMPrQELw0QNB8dsffsqcdQemjPB0w+moLcPh0JrKrHJ9hjBzdMPpcXTH7XRwwOk='
 
 
 class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
     def __init__(self, *args, **kwargs):
         FancyURLopener.__init__(self, *args, **kwargs)
+        # Headers to add to every request.
         # XXX: IMDb's web server doesn't like urllib-based programs,
         #      so lets fake to be Mozilla.
         #      Wow!  I'm shocked by my total lack of ethic! <g>
-        self.addheaders = [('User-agent', 'Mozilla/5.0')]
-        # This class is used also to perform "Exact Primary [Title|Name]"
-        # searches, and so by default the cookie is set.
+        self.set_header('User-agent', 'Mozilla/5.0')
+        # XXX: This class is used also to perform "Exact Primary
+        #      [Title|Name]" searches, and so by default the cookie is set.
         c_header = 'id=%s; uu=%s' % (_cookie_id, _cookie_uu)
-        self.addheaders.append(('Cookie', c_header))
+        self.set_header('Cookie', c_header)
+
+    def get_proxy(self):
+        """Return the used proxy, or an empty string."""
+        return self.proxies.get('http', '')
+
+    def set_proxy(self, proxy):
+        """Set the proxy."""
+        if not proxy:
+            if self.proxies.has_key('http'):
+                del self.proxies['http']
+        else:
+            if not proxy.lower().startswith('http://'):
+                proxy = 'http://%s' % proxy
+            self.proxies['http'] = proxy
+
+    def set_header(self, header, value, _overwrite=True):
+        """Set a default header."""
+        if _overwrite:
+            self.del_header(header)
+        self.addheaders.append((header, value))
+
+    def del_header(self, header):
+        """Remove a default header."""
+        for index in xrange(len(self.addheaders)):
+            if self.addheaders[index][0] == header:
+                del self.addheaders[index]
+                break
 
     def retrieve_unicode(self, url, size=-1):
         """Retrieves the given URL, and returns a unicode string,
@@ -84,11 +120,15 @@ class IMDbURLopener(FancyURLopener):
         encode = None
         try:
             if size != -1:
-                self.addheader('Range', 'bytes=0-%d' % size)
+                self.set_header('Range', 'bytes=0-%d' % size)
             uopener = self.open(url)
-            content = uopener.read(size=size)
+            kwds = {}
+            if PY_VERSION > (2, 3):
+                kwds['size'] = size
+            content = uopener.read(**kwds)
+            # Maybe the server is so nice to tell us the charset...
             server_encode = uopener.info().getparam('charset')
-            # look at the content-type HTML meta tag.
+            # Otherwise, look at the content-type HTML meta tag.
             if server_encode is None and content:
                 first_bytes = content[:512]
                 begin_h = first_bytes.find('text/html; charset=')
@@ -104,19 +144,18 @@ class IMDbURLopener(FancyURLopener):
                     pass
             uopener.close()
             if size != -1:
-                for index in xrange(len(self.addheaders)):
-                    if self.addheaders[index][0] == 'Range':
-                        del self.addheaders[index]
+                self.del_header('Range')
             self.close()
         except IOError, e:
             if size != -1:
-                for index in xrange(len(self.addheaders)):
-                    if self.addheaders[index][0] == 'Range':
-                        del self.addheaders[index]
+                # Ensure that the Range header is removed.
+                self.del_header('Range')
             raise IMDbDataAccessError, {'errcode': e.errno,
                                         'errmsg': str(e.strerror),
                                         'url': url,
-                                        'proxy': self.proxies.get('http', '')}
+                                        'proxy': self.get_proxy(),
+                                        'exception type': 'IOError',
+                                        'original exception': e}
         if encode is None:
             encode = 'latin_1'
             # The detection of the encoding is error prone...
@@ -130,14 +169,19 @@ class IMDbURLopener(FancyURLopener):
                                     'errcode': errcode,
                                     'errmsg': errmsg,
                                     'headers': headers,
-                                    'proxy': self.proxies.get('http', '')}
+                                    'error type': 'http_error_default',
+                                    'proxy': self.get_proxy()}
+
     def open_unknown(self, fullurl, data=None):
         raise IMDbDataAccessError, {'fullurl': fullurl,
                                     'data': str(data),
-                                    'proxy': self.proxies.get('http', '')}
+                                    'error type': 'open_unknown',
+                                    'proxy': self.get_proxy()}
+
     def open_unknown_proxy(self, proxy, fullurl, data=None):
         raise IMDbDataAccessError, {'proxy': str(proxy),
                                     'fullurl': fullurl,
+                                    'error type': 'open_unknown_proxy',
                                     'data': str(data)}
 
 
@@ -146,28 +190,27 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     accessSystem = 'http'
 
-    urlOpener = IMDbURLopener()
-
     def __init__(self, isThin=0, adultSearch=1, proxy=-1,
                 *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
+        self.urlOpener =  IMDbURLopener()
         # When isThin is set, we're parsing the "maindetails" page
         # of a movie (instead of the "combined" page) and movie/person
         # references are not collected if no defaultModFunct is provided.
         self.isThin = isThin
+        self._getRefs = True
+        self._mdparse = False
         if isThin:
             self.accessSystem = 'httpThin'
-            movie_parser.mdparse = 1
+            self._mdparse = True
             if self._defModFunct is None:
-                ParserBase.getRefs = 0
+                self._getRefs = False
                 from imdb.utils import modNull
                 self._defModFunct = modNull
-        else:
-            movie_parser.mdparse = 0
-            ParserBase.getRefs = 1
         self.do_adult_search(adultSearch)
-        if proxy != -1: self.set_proxy(proxy)
+        if proxy != -1:
+            self.set_proxy(proxy)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -195,6 +238,10 @@ class IMDbHTTPAccessSystem(IMDbBase):
         """
         return personID
 
+    def get_proxy(self):
+        """Return the used proxy or an empty string."""
+        return self.urlOpener.get_proxy()
+
     def set_proxy(self, proxy):
         """Set the web proxy to use.
 
@@ -203,17 +250,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
         If set, the value of the environment variable HTTP_PROXY is
         automatically used.
         """
-        if not proxy:
-            if self.urlOpener.proxies.has_key('http'):
-                del self.urlOpener.proxies['http']
-        else:
-            if not proxy.lower().startswith('http://'):
-                proxy = 'http://%s' % proxy
-            self.urlOpener.proxies['http'] = proxy
-
-    def get_proxy(self):
-        """Return the used proxy or an empty string."""
-        return self.urlOpener.proxies.get('http', '')
+        self.urlOpener.set_proxy(proxy)
 
     def do_adult_search(self, doAdult,
                         cookie_id=_cookie_id, cookie_uu=_cookie_uu):
@@ -221,13 +258,11 @@ class IMDbHTTPAccessSystem(IMDbBase):
         search results; cookie_id and cookie_uu are optional
         parameters to select a specific account (see your cookie
         or cookies.txt file."""
-        for index in xrange(len(self.urlOpener.addheaders)):
-            if self.urlOpener.addheaders[index][0] == 'Cookie':
-                del self.urlOpener.addheaders[index]
-                break
         if doAdult:
             c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
-            self.urlOpener.addheaders += [('Cookie', c_header)]
+            self.urlOpener.set_header('Cookie', c_header)
+        else:
+            self.urlOpener.del_header('Cookie')
 
     def _retrieve(self, url, size=-1):
         """Retrieve the given URL."""
@@ -248,10 +283,6 @@ class IMDbHTTPAccessSystem(IMDbBase):
         # The retrieved page contains no results, because too many
         # titles or names contain the string we're looking for.
         params = 'q=%s;more=%s' % (quote_plus(ton), kind)
-        ##if kind == 'nm':
-        ##    params = 'q=%s;more=nm' % quote_plus(ton)
-        ##else:
-        ##    params = 'q=%s;more=tt' % quote_plus(ton)
         size = 22528 + results * 512
         return self._retrieve(imdbURL_search % params, size=size)
 
@@ -270,7 +301,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
             cont = self._retrieve(imdbURL_movie % movieID + 'combined')
         else:
             cont = self._retrieve(imdbURL_movie % movieID + 'maindetails')
-        return movie_parser.parse(cont)
+        return movie_parser.parse(cont, mdparse=self._mdparse)
 
     def get_movie_full_credits(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'fullcredits')
@@ -278,7 +309,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_plot(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'plotsummary')
-        return plot_parser.parse(cont)
+        return plot_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_awards(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'awards')
@@ -294,19 +325,19 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_alternate_versions(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'alternateversions')
-        return alternateversions_parser.parse(cont)
+        return alternateversions_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_crazy_credits(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'crazycredits')
-        return crazycredits_parser.parse(cont)
+        return crazycredits_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_goofs(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'goofs')
-        return goofs_parser.parse(cont)
+        return goofs_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_quotes(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'quotes')
-        return quotes_parser.parse(cont)
+        return quotes_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_release_dates(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'releaseinfo')
@@ -322,7 +353,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_trivia(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'trivia')
-        return trivia_parser.parse(cont)
+        return trivia_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_connections(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'movieconnections')
@@ -334,11 +365,11 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_business(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'business')
-        return business_parser.parse(cont)
+        return business_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_literature(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'literature')
-        return tech_parser.parse(cont)
+        return literature_parser.parse(cont)
 
     def get_movie_locations(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'locations')
@@ -350,7 +381,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_dvd(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'dvd')
-        return dvd_parser.parse(cont)
+        return dvd_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_recommendations(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'recommendations')
@@ -382,7 +413,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_news(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'news')
-        return news_parser.parse(cont)
+        return news_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_amazon_reviews(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'amazon')
@@ -390,7 +421,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_guests(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'epcast')
-        return guests_parser.parse(cont)
+        return episodes_parser.parse(cont)
+    get_movie_episodes_cast = get_movie_guests
 
     def get_movie_merchandising_links(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'sales')
@@ -423,12 +455,20 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_faqs(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'faq')
-        return movie_faqs_parser.parse(cont)
+        return movie_faqs_parser.parse(cont, getRefs=self._getRefs)
 
     def get_movie_airing(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'tvschedule')
         return airing_parser.parse(cont)
 
+    def get_movie_synopsis(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'synopsis')
+        return synopsis_parser.parse(cont)
+
+    def get_movie_parents_guide(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'parentalguide')
+        return parentsguide_parser.parse(cont)
+
     def _search_person(self, name, results):
         # The URL of the query.
         # XXX: To retrieve the complete results list:
@@ -450,7 +490,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_person_biography(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'bio')
-        return bio_parser.parse(cont)
+        return bio_parser.parse(cont, getRefs=self._getRefs)
 
     def get_person_awards(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'awards')
@@ -458,7 +498,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_person_other_works(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'otherworks')
-        return otherworks_parser.parse(cont)
+        return otherworks_parser.parse(cont, getRefs=self._getRefs)
 
     def get_person_agent(self, personID):
         cont = self._retrieve(imdbURL_person % personID + 'agent')
@@ -480,4 +520,16 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person % personID + 'filmoseries')
         return person_series_parser.parse(cont)
 
+    def get_person_merchandising_links(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'forsale')
+        return sales_parser.parse(cont)
+
+    def get_person_genres_links(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'filmogenre')
+        return person_genres_parser.parse(cont)
+
+    def get_person_keywords_links(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'filmokey')
+        return person_keywords_parser.parse(cont)
+
 
